;________________________________________________________________________
;
;                             P I C - E . A S M
;________________________________________________________________________
;
; PIC-E Position Sending  by Byon Garrabrant  N6BG  byon@mail.com  Feb 99
;________________________________________________________________________
;
;   This software is  available only for non-commercial  amateur radio 
;   or educational  applications.   ALL  other  uses  are  prohibited.  
;   This  software may  be  modified  only  if the  resulting code  be 
;   made available  publicly  and the  original  author given  credit.  
;   Contact the author or TAPR for information on commercial licenses. 
;   (c) Copyright 1999,  Byon Garrabrant,  N6BG.  All rights reserved. 
;________________________________________________________________________

	LIST	P=PIC16F84
	include	"P16F84.INC"
	__CONFIG _HS_OSC & _CP_OFF & _PWRTE_ON & _WDT_OFF
;________________________________________________________________________

	cblock	H'0C'			; system variables
		beacon_counter
		temp
		temp2
	endc
;________________________________________________________________________

#define	PTT_OUT		PORTA, 2
;________________________________________________________________________

	ORG	0			; start code
	goto	main
;________________________________________________________________________

	include	"OPTIONS.ASM"		; include options code
	include	"MX614.ASM"		; include MX614 Modem code
	include	"NMEA.ASM"		; include NMEA code
	include	"AX25.ASM"		; include AX.25 UI code
	include	"SERIAL.ASM"		; include serial code
	include	"TIMING.ASM"		; include timing code
;________________________________________________________________________

 if 0 
Serial_Send_Space
	movlw	' '
Serial_Send_Byte
	goto	serialTransmit

Serial_Send_Hex
	movwf	temp
	swapf	temp, W
	call	Serial_Send_Hex_Nibble
	movfw	temp
;	goto	Serial_Send_Hex_Nibble

Serial_Send_Hex_Nibble
	andlw	H'0F'
	sublw	9
	skpc
	 addlw	-7
	sublw	'0'+9
	goto	Serial_Send_Byte
 endif
;________________________________________________________________________

AX25_Send_Space
	movlw	' '
AX25_Send_Byte
	goto	AX25_Send_Data

AX25_Send_Hex
	movwf	temp
	swapf	temp, W
	call	AX25_Send_Hex_Nibble
	movfw	temp
;	goto	Serial_Send_Hex_Nibble

AX25_Send_Hex_Nibble
	andlw	H'0F'
	sublw	9
	skpc
	 addlw	-7
	sublw	'0'+9
	goto	AX25_Send_Byte
;________________________________________________________________________

Packet_Send
	bsf	PTT_OUT			; press PTT
	call	AX25_Send_Start

	movlw	MIC_E_DEST_BASE 	; find the current offset
	movwf	FSR		
Packet_Send_Dest_Loop
	movfw	INDF
	call	AX25_Send_Data

	incf	FSR, F
	movfw	FSR			; check if we have receive
	xorlw	MIC_E_DEST_BASE + MIC_E_DEST_LEN ; all chars of the string
	skpz				; if not
	 goto	Packet_Send_Dest_Loop

	movlw	OPT_CALLSIGN_DIGI
	movwf	temp
Packet_Send_Callsign
	movfw	temp
	call	Get_Option
	iorlw	0
	skpnz
	 goto	Packet_Send_Type

	movwf	temp2
	incf	temp, W
	call	Get_Option
	iorlw	0
	clrc
	skpnz
	 setc
 	rlf	temp2, W

	call	AX25_Send_Data
	incf	temp, F
	goto	Packet_Send_Callsign

Packet_Send_Type

	movlw	H'03'
	call	AX25_Send_Data
	movlw	H'F0'
	call	AX25_Send_Data

Packet_Send_Info_Loop
	movfw	INDF
	call	AX25_Send_Data

	incf	FSR, F
	movfw	FSR			; check if we have receive
	xorlw	MIC_E_INF_BASE + MIC_E_INF_LEN	; all chars of the string
	skpz				; if not
	 goto	Packet_Send_Info_Loop

	movlw	OPT_SYMBOL		; get and send the symbol
	call	Get_Option
	call	AX25_Send_Data

	movlw	OPT_SYMBOL_TABLE	; get and send the symbol table
	call	Get_Option
	call	AX25_Send_Data

	decfsz	beacon_counter, F
	 goto	Packet_Beacon_Done

	movlw	OPT_BEACON_RATE	; get the beacon rate
	call	Get_Option
	movwf	beacon_counter

	movlw	OPT_BEACON_OFFSET
	call	Get_Option
	movwf	temp
Packet_Send_Beacon
	movfw	temp
	call	Get_Option
	iorlw	0
	skpnz
	 goto	Packet_Beacon_Done
	call	AX25_Send_Data
	incf	temp, F
	goto	Packet_Send_Beacon

Packet_Beacon_Done

 if 0
Packet_Send_Debug
	call	AX25_Send_Space

	movlw	NMEA_STRING_BASE	; find the current offset
	movwf	FSR		
Packet_Send_Debug_1
	movfw	INDF
	call	AX25_Send_Data

	incf	FSR, F
	movfw	FSR			; check if we have receive
	xorlw	NMEA_STRING_BASE + NMEA_STRING_LEN
	skpz				; if not
	 goto	Packet_Send_Debug_1

; return 

	call	AX25_Send_Space

Packet_Send_Debug_2
	movfw	INDF
	call	AX25_Send_Hex
	call	AX25_Send_Space

	incf	FSR, F
	movfw	FSR			; check if we have receive
	xorlw	MIC_E_INF_BASE + MIC_E_INF_LEN
	skpz				; if not
	 goto	Packet_Send_Debug_2
 endif

	call	AX25_Send_End
	bcf	PTT_OUT			; release PTT
	return
;________________________________________________________________________

main
	call	serialInit
	call	NMEA_Init
	call	Tone_Init
	call	Timing_Init

	bsf	STATUS, RP0		; Select Bank 1
	bcf	PTT_OUT			; make PTT an output
	bcf	VALID_OUT
	bcf	STATUS, RP0		; Select Bank 0
	bcf	PTT_OUT			; release PTT

	movlw	1
	movwf	beacon_counter

mainLoop

	call	serialReceive		; receive serial
	iorlw	0			; look at result
	skpz				; if non zero
	 call	NMEA_Decode		;  send to NMEA decoder

	call	Timing_Check		; check timing and inputs
	iorlw	0			; look at result
	skpz				; if non zero
	 call	Packet_Send		; send a packet

	goto	mainLoop		; loop back again

	END
;________________________________________________________________________

;                                        spd   ber
;$GPRMC,083036,A,3610.999,N,11516.337,W,219.6,326.6,221197,013.7,E*6F
;$GPRMC,072004,A,5124.164,N,00039.253,W,021.7,028.9,180496,004.1,W*78 
;$GPRMC,000000,A,3046.993,N,08140.503,W,100.0,100.0,090798,000.0,W*78 

