;________________________________________________________________________
;
;                            T I M I N G . A S M
;________________________________________________________________________
;
; Timing management code  by Byon Garrabrant  N6BG  byon@mail.com  Feb 99
;________________________________________________________________________
;
;   This software is  available only for non-commercial  amateur radio 
;   or educational  applications.   ALL  other  uses  are  prohibited.  
;   This  software may  be  modified  only  if the  resulting code  be 
;   made available  publicly  and the  original  author given  credit.  
;   Contact the author or TAPR for information on commercial licenses. 
;   (c) Copyright 1999,  Byon Garrabrant,  N6BG.  All rights reserved. 
;________________________________________________________________________

#define	PTT_IN		PORTA, 3
#define	VOX_IN		PORTB, 3
#define	VALID_OUT	PORTB, 0
#define	VALID_TIME	D'15'
;________________________________________________________________________

	cblock
		lastTMR0
		sub_second_timer
		ten_second_timer
		quiet_timer
 		transmission_timer
		ptt_in_history
	endc
;________________________________________________________________________

Timing_Init
	movlw	D'1'
	movwf	ten_second_timer
	movwf	sub_second_timer
 	clrf	NMEA_valid_timer	; init valid timer to 255
 	decf	NMEA_valid_timer, F
	clrf	quiet_timer
	clrf	transmission_timer
	clrf	ptt_in_history
	return

Timing_Check_Update

	btfsc	VOX_IN			; clear quiet_timer
	 clrf	quiet_timer		; if voice activity

	movfw	lastTMR0		; get difference from
	subwf	TMR0, W			; last time and
	addwf	lastTMR0, F		; add to last time
	skpc				; if no wrap
	 return				; leave

	clrc				; update ptt history
	btfss	PTT_IN
	 setc
	rlf	ptt_in_history, F

	decfsz	sub_second_timer, F	; count down the sub seconds
	 return				; if not zero, leave

	movlw	D'38'			; reset the sub seconds
;	movlw	D'2'			; TWENTY X SPEEDUP
	movwf	sub_second_timer	
	incf	quiet_timer, F		; bump quiet_timer
	skpnz				; but clamp
	 decf	quiet_timer, F		; at 255
	btfsc	VOX_IN			; clear quiet_timer
	 clrf	quiet_timer		; if voice activity

	incf	NMEA_valid_timer, F	; inc valid_timer
	skpnz				; but clamp
	 decf	NMEA_valid_timer, F	; to 255

	decfsz	ten_second_timer, F	; count down ten seconds
	 return				; if not done, leave

	movlw	D'10'			; reset the ten second timer
	movwf	ten_second_timer
	incf	transmission_timer, F	; inc transmission_timer
	skpnz				; but clamp
	 decf	transmission_timer, F	; to 255

	return
	

Timing_Check
	call	Timing_Check_Update

	movfw	NMEA_valid_timer	; update the valid light
	sublw	VALID_TIME
	skpnc
	 goto	Timing_Valid
Timing_Invalid
	bcf	VALID_OUT

	movfw	MIC_E_INF_BASE + 0	; if data was valid,
	xorlw	VALID_FLAG		; 
	movlw	INVALID_FLAG		
	skpnz
	 movwf	MIC_E_INF_BASE + 0 	; change to invalid
	goto	Timing_Continue
Timing_Valid
	bsf	VALID_OUT

Timing_Continue
					; first check for manual
	movlw	OPT_MANUAL_TIME		; check manual time
	call	Get_Option
	subwf	transmission_timer, W
	bnc	Check_Auto

	btfsc	ptt_in_history, 0	; if ptt in down
	 bsf	PTT_OUT			; press PTT out

	movfw	ptt_in_history
;	andlw	H'3'
;	xorlw	H'2'
	xorlw	H'FE'
	bz	Do_Transmit_Manual

Check_Auto

	tstf	ptt_in_history		; if no PTT IN recently
	bnz	Check_Done

	movlw	OPT_AUTO_TIME		; check auto time
	call	Get_Option
	subwf	transmission_timer, W
	bnc	Check_Done

	movlw	OPT_QUIET_TIME		; check quiet time
	call	Get_Option
	subwf	quiet_timer, W
	bnc	Check_Done

Do_Transmit_Auto
	call	AX25_Prep_Auto
	clrf	transmission_timer
	call	NMEA_Reset
	retlw	1

Do_Transmit_Manual
	call	AX25_Prep_Manual
	clrf	transmission_timer
	call	NMEA_Reset
	retlw	1

Check_Done	
	retlw	0

;________________________________________________________________________
